@@ -1,3 +1,5 @@
+use std::fmt::Debug;
+
 ///
 /// Common formatting for Token and OwnedToken
 ///
@@ -5,5 +7,7 @@ pub trait FormatToken {
     ///
     /// Generates a formatted position which an editor can follow via mouse click.
     ///
-    fn format(&self, file_name: &str, terminal_names: &'static [&'static str]) -> String;
+    fn format<T>(&self, file_name: &T, terminal_names: &'static [&'static str]) -> String
+    where
+        T: Debug;
 }
